@@ -6,23 +6,23 @@ require Exporter;
 our @ISA= 'Exporter';
 
 my %lookup= (
-    0  => [\&p2c_string, \&c2p_string, 'TYPE_CUSTOM'],
-    1  => [\&p2c_string, \&c2p_string, 'TYPE_ASCII'],
-    2  => [\&p2c_bigint, \&c2p_bigint, 'TYPE_BIGINT'],
-    3  => [\&p2c_string, \&c2p_string, 'TYPE_BLOB'],
-    4  => [\&p2c_bool,   \&c2p_bool,   'TYPE_BOOLEAN'],
-    5  => [\&not_impl,   \&not_impl,   'TYPE_COUNTER'],
-    6  => [\&not_impl,   \&not_impl,   'TYPE_DECIMAL'],
-    7  => [\&p2c_double, \&c2p_double, 'TYPE_DOUBLE'],
-    8  => [\&p2c_float,  \&c2p_float,  'TYPE_FLOAT'],
-    9  => [\&p2c_int,    \&c2p_int,    'TYPE_INT'],
-    10 => [\&p2c_string, \&c2p_string, 'TYPE_TEXT'],
-    11 => [\&not_impl,   \&not_impl,   'TYPE_TIMESTAMP'],
-    12 => [\&not_impl,   \&not_impl,   'TYPE_UUID'],
-    13 => [\&p2c_string, \&c2p_string, 'TYPE_VARCHAR'],
-    14 => [\&not_impl,   \&not_impl,   'TYPE_VARINT'],
-    15 => [\&not_impl,   \&not_impl,   'TYPE_TIMEUUID'],
-    16 => [\&not_impl,   \&not_impl,   'TYPE_INET'],
+    0  => [\&p2c_string, \&c2p_string,      'TYPE_CUSTOM'],
+    1  => [\&p2c_string, \&c2p_string,      'TYPE_ASCII'],
+    2  => [\&p2c_bigint, \&c2p_bigint,      'TYPE_BIGINT'],
+    3  => [\&p2c_string, \&c2p_string,      'TYPE_BLOB'],
+    4  => [\&p2c_bool,   \&c2p_bool,        'TYPE_BOOLEAN'],
+    5  => [\&not_impl,   \&not_impl,        'TYPE_COUNTER'],
+    6  => [\&not_impl,   \&not_impl,        'TYPE_DECIMAL'],
+    7  => [\&p2c_double, \&c2p_double,      'TYPE_DOUBLE'],
+    8  => [\&p2c_float,  \&c2p_float,       'TYPE_FLOAT'],
+    9  => [\&p2c_int,    \&c2p_int,         'TYPE_INT'],
+    10 => [\&p2c_string, \&c2p_utf8string,  'TYPE_TEXT'],
+    11 => [\&not_impl,   \&not_impl,        'TYPE_TIMESTAMP'],
+    12 => [\&not_impl,   \&not_impl,        'TYPE_UUID'],
+    13 => [\&p2c_string, \&c2p_utf8string,  'TYPE_VARCHAR'],
+    14 => [\&not_impl,   \&not_impl,        'TYPE_VARINT'],
+    15 => [\&not_impl,   \&not_impl,        'TYPE_TIMEUUID'],
+    16 => [\&not_impl,   \&not_impl,        'TYPE_INET'],
 );
 
 sub not_impl { ... }
@@ -40,6 +40,7 @@ sub p2c_string {
     return ("pack('l>/a', \$_[$i])", "utf8::encode \$_[$i]");
 }
 sub c2p_string { return shift }
+sub c2p_utf8string { my $var= shift; return ($var, "utf8::decode $var") }
 sub p2c_bigint { return   _pack('q>', 8, @_) }
 sub c2p_bigint { return _unpack('q>', 8, @_) }
 sub p2c_int { return   _pack('l>', 4, @_) }
@@ -89,10 +90,13 @@ sub build_row_decoder {
     for my $type (@$types) {
         if (ref $type) { $type= $type->{type}; }
         my $t= $lookup{$type} or die "Unknown type $type";
+        my ($c, $prep)= $t->[1]('$b');
+
         $code .= '    $a= unpack("l>", substr $_[0], 0, 4, "");'."\n";
         $code .= '    if ($a >= 0) {'."\n";
         $code .= '        $b= substr $_[0], 0, $a, "";'."\n";
-        $code .= '        push @row, ('.$t->[1]('$b').');'."\n";
+        $code .= '        '.$prep.';'."\n" if $prep;
+        $code .= '        push @row, ('.$c.');'."\n";
         $code .= '    } else { push @row, undef; }'."\n";
         $i++;
     }
